#include "weather.h"
#include "ui_weather.h"

#include <QtCharts/QAreaSeries>
#include <QtCharts/QChartView>
#include <QtCharts/QLineSeries>
#include <QtCharts/QDateTimeAxis>

#include <QNetworkAccessManager>
#include <QtNetwork/QNetworkReply>
#include <QtNetwork/QNetworkRequest>
#include <QEventLoop>

#include <QJsonDocument>
#include <QJsonObject>

#include <QImage>

weather::weather(QWidget *parent)
    : QMainWindow(parent)
    , ui(new Ui::weather)
{
    ui->setupUi(this);

    this->forecastView();
    this->on_btnUpdate_clicked();
}

weather::~weather()
{
    delete ui;
}

void weather::forecastView(void){

    QNetworkAccessManager *mgr = new QNetworkAccessManager;
    QNetworkRequest req(QString("https://api.openweathermap.org/data/2.5/forecast?q=prague&appid=9015878572d5a56d22bafb7d1e71c92e"));
    QNetworkReply *reply = mgr->get(req);
    QEventLoop evtLoop;
    QObject::connect(mgr, SIGNAL(finished(QNetworkReply*)), &evtLoop, SLOT(quit()));

    evtLoop.exec();

    if(reply->error() == QNetworkReply::NoError){
        QString data = reply->readAll();
        QJsonDocument json = QJsonDocument::fromJson(data.toUtf8());
        QJsonObject   jsonObject = json.object();
        int listCount = jsonObject["cnt"].toInt();

        QLineSeries *dataActual = new QLineSeries();
        dataActual->setName("Teplota");

        // nacteni a vykresleni vsech hodnot v JSON souboru
        for(int i=0; i<listCount; i++){
            // todo prochazet data v JSON souboru
            int cas = i;
            double teplota = 22 + i;
            QDateTime time = QDateTime::fromSecsSinceEpoch(cas);
            *dataActual << QPointF(time.toSecsSinceEpoch(), teplota);
        }

        // nastaveni osy X
        QDateTimeAxis *dateTimeAxis = new QDateTimeAxis;
        dateTimeAxis->setTickCount(7);
        dateTimeAxis->setFormat("dd.MM hh:mm");
        dateTimeAxis->setTitleText("Datum");

        // definice min a max pro osu X
        QJsonObject   jsonListMin = jsonObject["list"][0].toObject();
        QDateTime timeMin = QDateTime::fromSecsSinceEpoch(jsonListMin["dt"].toInt());
        QJsonObject   jsonListMax = jsonObject["list"][listCount-1].toObject();
        QDateTime timeMax = QDateTime::fromSecsSinceEpoch(jsonListMax["dt"].toInt());
        dateTimeAxis->setMin(timeMin.addSecs(0));
        dateTimeAxis->setMax(timeMax.addSecs(0));

        QAreaSeries *series = new QAreaSeries(dataActual);
        QPen pen(0x059605);
        pen.setWidth(3);
        series->setPen(pen);

        QChart *chart = new QChart();
        chart->addSeries(dataActual);
        chart->setTitle(QString("Predpoved Pocasi"));
        chart->createDefaultAxes();
        chart->axisX()->setVisible(false);
        chart->addAxis(dateTimeAxis, Qt::AlignBottom);
        //chart->axisX()->setRange(timeMin.addSecs(-3600), timeMax.addSecs(3600));

        chart->axisY()->setTitleText("Teplota");
        ui->chartForecast->setChart(chart);
    }
}

void weather::getIcon(QString icon){
    QNetworkAccessManager *mgr = new QNetworkAccessManager;
    QNetworkRequest req(QString("http://openweathermap.org/img/wn/%1@2x.png").arg(icon));
    QNetworkReply *reply = mgr->get(req);
    QEventLoop evtLoop;
    QObject::connect(mgr, SIGNAL(finished(QNetworkReply*)), &evtLoop, SLOT(quit()));

    evtLoop.exec();

    if(reply->error() == QNetworkReply::NoError){
        // todo nacist obrazek jako pixmap
    }
}

void weather::on_btnUpdate_clicked()
{
    /* todo:
     *  1, nacist data z https://api.openweathermap.org/data/2.5/weather?q=prague&appid=9015878572d5a56d22bafb7d1e71c92e
     *  2, overit format dat v json souboru
     *  3, vytvorit json objekty pro cteni dat
     *  4, zobrazit je v labelech na UI
     */
}

